package circuit;

import java.io.*;
import java_cup.runtime.*;
import node.*;
import node.stmtNode.*;
import text.*;
import grammar.*;

import runEnv.*;

public class Main {

	public static boolean debug = false;
	public static boolean expand = false;
	public static boolean print = false;

	public static void main( String[] argv ) {
		String dirName = null;

		try {
			for ( int i = 0; i < argv.length; i++ ) {
				if ( argv[ i ].equals( "-debug" ) ) {
					debug = true;
					}
				else if ( argv[ i ].equals( "-expand" ) ) {
					expand = true;
					}
				else if ( argv[ i ].equals( "-print" ) ) {
					print = true;
					}
				else if ( argv[ i ].equals( "-dir" ) ) {
					i++;
					if ( i >= argv.length )
						throw new Error( "Missing directory name" );
					dirName = argv[ i ];
					}
				else {
					throw new Error( 
						"Invalid argument: \"" + argv[ i ] 
						+ "\"\nUsage: java Main [-debug] [-expand] [-print] -dir directory" );
					}
				}

			if ( dirName == null )
				throw new Error( "Directory not specified" );

			Print.setError( new File( dirName, "program.err" ) );
			Print.setReprint( new File( dirName, "program.print" ) );
			Print.setInterp( new File( dirName, "program.out" ) );

			parser libraryParser = new parser( 
				new File( "LIBRARY", "globalLibrary.in" ) );
			DeclStmtListNode globalLibrary = 
				( DeclStmtListNode ) libraryParser.parse().value;

			parser programParser = new parser( 
				new File( dirName, "program.in" ) );
			DeclStmtListNode declStmtList = 
				( DeclStmtListNode ) programParser.parse().value;

			ProgramNode program = new ProgramNode( globalLibrary, declStmtList );

			Print.error().println( "Reprinting ... " );
			Print.reprint().print( program );
			if ( print )
				System.exit( 0 );
			Print.error().println( "Evaluate ... " );
			try {
				program.eval();
				}
			catch ( Error exception ) {
				Print.error().println( "User Error:\n\n" + exception.getMessage() );
				Print.error().printStackTrace( exception );
				}
			}
		catch ( Throwable exception ) {
			exception.printStackTrace();
			Print.error().println( "Exception in Main:\n\n" + exception );
			Print.error().printStackTrace( exception );
			System.exit( -1 );
			}
		}
	}

