package runEnv;

import runEnv.basicValue.*;
import builtin.*;

public class PathArrayValue extends RunValue {

	private PathValue[] base;
	private PathValue[] base() { return base; }
	
	private int offset;
	private int offset() { return offset; }

	private int size;	
	public int size() { return size; }

	public PathValue elementAt( int i ) { 
		if ( i < 0 || i >= size )
			throw new Error( "Invalid index of " + i + " for PathArrayValue " + this );
		return base[ offset + i ]; 
		}
	
	public PathArrayValue( int size ) {
		if ( size < 0 )
			throw new Error( 
				"Invalid PathArrayValue( " + size + " )" );
		base = new PathValue[ size ];
		this.size = size;
		this.offset = 0;
		for ( int i = 0; i < size; i++ )
			base[ i ] = new PathValue();
//		System.err.println( "Creating PathArrayValue( " + size + " ) " + this );
		}

	private PathArrayValue( PathValue[] base, int offset, int size ) {
		if ( offset < 0 || size < 0 || offset + size > base.length )
			throw new Error( 
				"Invalid PathArrayValue( base " + base.length + ", " + offset + ", " + size + " )" );
		this.base = base;
		this.offset = offset;
		this.size = size;
//		System.err.println( "Creating PathArrayValue( " + offset + ", " + size + " ) " + this );
		}

	public PathArrayValue( PathArrayValue[] value ) {
		size = 0;
		for ( int i = 0; i < value.length; i++ ) {
			PathArrayValue element = value[ i ];
			size += element.size();
			}
		base = new PathValue[ size ];
		int k = 0;
		for ( int i = value.length - 1; i >= 0; --i ) {
			PathArrayValue element = value[ i ];
			for ( int j = 0; j < element.size(); j++ )
				base[ k++ ] = element.elementAt( j );
			}
/*
		System.err.println( "Creating PathArrayValue " + this + " from " );
		for ( int i = 0; i < value.length; i++ ) {
			PathArrayValue element = value[ i ];
			System.err.println( "\t" + element );
			}
*/
		}
		
	public PathArrayValue pathArrayValue() {
		return this;
		}

	public boolean isPathArray() {
		return true;
		}

	public boolean equals( Object other ) {
		if ( other == null )
			return false;
		if ( ! ( other instanceof RunValue ) )
			return false;
		RunValue runValue = ( RunValue ) other;
		if ( ! ( runValue instanceof PathArrayValue ) )
			return false;
		PathArrayValue otherArrayValue = ( PathArrayValue ) runValue;
		return base == otherArrayValue.base() 
			&& offset == otherArrayValue.offset() 
			&& size == otherArrayValue.size();
		}

	public String toString() {
		String result = "";
		if ( size() == 0 )
			return "{}";
		for ( int i = 0; i < size(); i++ ) {
			if ( i > 0 )
				result += ", ";
			result += elementAt( i );
			}
		return "{ " + result + " }";
		}

	public PathArrayValue index( int start ) {
		if ( start < 0 || start >= size )
			throw new Error( "index out of bounds" );
		return new PathArrayValue( base, this.offset + start, 1 );
		}

	public PathArrayValue subArray( int start, int size ) {
		if ( start < 0 || size < 0 || start + size > this.size )
			throw new Error( "subarray " + this 
				+ "[ " + size + " @ " + start + " ] out of bounds" );
		return new PathArrayValue( base, this.offset + start, size );
		}
		
	public long getValue() {
		long value = 0;
		for ( int i = 0; i < size(); i++ ) {
			if ( elementAt( i ).getValue() )
				value |= 1L << i;
			}
//		System.err.println( "getValue " + Long.toString( value, 16 ) );
		return value;
		}
	
	public void setValue( long value ) {
//		System.err.println( "setValue " + Long.toString( value, 16 ) );
		for ( int i = 0; i < size(); i++ ) {
			elementAt( i ).setValue( ( value & ( 1L << i ) ) != 0 );
			}
		}

	public void setValue( PathArrayValue otherArrayValue ) {
//		System.err.println( "setValue " + otherArrayValue );
		for ( int i = 0; i < size(); i++ ) {
			boolean value = otherArrayValue.elementAt( i ).getValue();
			elementAt( i ).setValue( value );
			}
		}

	public void addListener( Component component ) {
		for ( int i = 0; i < size(); i++ ) {
			PathValue  path = elementAt( i );
			path.addListener( component );
			}
		}

	}
